@@ -126,12 +126,12 @@ class CSVBufferIterator {
     }
 
     trailing_cr_ = (buf->data()[buf->size() - 1] == '\r');
-    buf = SliceBuffer(buf, offset);
+    buf = SliceBuffer(std::move(buf), offset);
     if (buf->size() == 0) {
       // EOF
       return TransformFinish();
     } else {
-      return TransformYield(buf);
+      return TransformYield(std::move(buf));
     }
   }
 
@@ -1053,8 +1053,8 @@ class AsyncThreadedTableReader
     auto self = shared_from_this();
     return ProcessFirstBuffer().Then([self](const std::shared_ptr<Buffer>& first_buffer) {
       auto block_generator = ThreadedBlockReader::MakeAsyncIterator(
-          self->buffer_generator_, MakeChunker(self->parse_options_),
-          std::move(first_buffer), self->read_options_.skip_rows_after_names);
+          self->buffer_generator_, MakeChunker(self->parse_options_), first_buffer,
+          self->read_options_.skip_rows_after_names);
 
       std::function<Status(CSVBlock)> block_visitor =
           [self](CSVBlock maybe_block) -> Status {
