@@ -20,12 +20,15 @@
 #include "arrow/util/config.h"  // IWYU pragma: export
 
 #include "arrow/filesystem/filesystem.h"  // IWYU pragma: export
-#include "arrow/filesystem/hdfs.h"        // IWYU pragma: export
-#ifdef ARROW_GCS
-#include "arrow/filesystem/gcsfs.h"  // IWYU pragma: export
+#ifdef ARROW_AZURE
+#  include "arrow/filesystem/azurefs.h"  // IWYU pragma: export
 #endif
+#ifdef ARROW_GCS
+#  include "arrow/filesystem/gcsfs.h"  // IWYU pragma: export
+#endif
+#include "arrow/filesystem/hdfs.h"     // IWYU pragma: export
 #include "arrow/filesystem/localfs.h"  // IWYU pragma: export
 #include "arrow/filesystem/mockfs.h"   // IWYU pragma: export
 #ifdef ARROW_S3
-#include "arrow/filesystem/s3fs.h"  // IWYU pragma: export
+#  include "arrow/filesystem/s3fs.h"  // IWYU pragma: export
 #endif
