@@ -17,6 +17,7 @@
 
 #include "arrow/filesystem/util_internal.h"
 
+#include <algorithm>
 #include <cerrno>
 
 #include "arrow/buffer.h"
@@ -29,7 +30,7 @@
 namespace arrow {
 
 using internal::StatusDetailFromErrno;
-using internal::Uri;
+using util::Uri;
 
 namespace fs {
 namespace internal {
@@ -63,11 +64,21 @@ Status PathNotFound(std::string_view path) {
       .WithDetail(StatusDetailFromErrno(ENOENT));
 }
 
+Status IsADir(std::string_view path) {
+  return Status::IOError("Is a directory: '", path, "'")
+      .WithDetail(StatusDetailFromErrno(EISDIR));
+}
+
 Status NotADir(std::string_view path) {
   return Status::IOError("Not a directory: '", path, "'")
       .WithDetail(StatusDetailFromErrno(ENOTDIR));
 }
 
+Status NotEmpty(std::string_view path) {
+  return Status::IOError("Directory not empty: '", path, "'")
+      .WithDetail(StatusDetailFromErrno(ENOTEMPTY));
+}
+
 Status NotAFile(std::string_view path) {
   return Status::IOError("Not a regular file: '", path, "'");
 }
@@ -92,7 +103,7 @@ Result<Uri> ParseFileSystemUri(const std::string& uri_string) {
     return status;
 #endif
   }
-  return std::move(uri);
+  return uri;
 }
 
 #ifdef _WIN32
